<?php

/**
* Clase manejadora de las recetas
*
* Maneja:
* creacion de recetas
* favorito de las recetas
* addicion de variantes/comentarios
*
* Ademas, ofrece:
* obtencion de todos los datos de una receta
* comprobar si es favorita
* obtencion de los ingredientes
* obtención de la imagen
* obtencion de las aportaciones
*
* otras funciones especiales (metodos de apoyo)
*
* Necesita: SqlRecipe
*/


class Nuts_Recipe extends Nuts_MongoRecipe
{

	function get_ingredients()
	{	
		return array(); //no se usa
		$ingr = Nuts_Recipe::get_ingredients_for($this->id);
		foreach ($ingr as &$item)
		{
			if (!$item['measure'])
				continue;
			if (substr($item['measure'], -3) == " de")
				continue;
			if (is_numeric($item['measure']))
				continue;
			if (preg_match('@^\d* ?\d+/\d+$@u', $item['measure'])) //     3 1/2
				continue;
			$item['measure'] = $item['measure'] . ' de';
		}
		
		function cmp($a, $b)
		{
		    if ($a == $b) {
		        return 0;
		    }
		    return ($a > $b) ? -1 : 1;
		}

		usort($ingr, "cmp");
		
		return $ingr;
	}

	function get_image()
	{
		return Nuts_Recipe::get_image_for($this->row['namerecipe'],$this->row['user']);
	}	
	
	function get_image_for($user, $id)
	{	
		$file="static/image/recipe/$user.$id.jpg";
		if (!file_exists( $file ))
			$file = "static/image/recipe/no.png";
		else
			$file.="?nocache";
		return $file;
	}
	
	function get_alteratives($user=null)
	{
		return Nuts_Recipe::get_alternatives_for($this->id);
	}
	
	function get_alternatives_for($id)
	{
		$r = parent::get_alternatives_for($id);
		foreach ($r as &$alternative)
		{
			$alternative['ingredients']['ins'] = parent::get_ins_for($alternative['idalternative'], $id);
			$alternative['ingredients']['del'] = parent::get_del_for($alternative['idalternative'], $id);
			//si los arrays estan vacios eliminarlos  para que $alternative['ingredients'] sea falso en un contexto booleano
			if(!$alternative['ingredients']['ins']) unset($alternative['ingredients']['ins']);
			if(!$alternative['ingredients']['del']) unset($alternative['ingredients']['del']);
		}
		return $r;
	}
	
	//modificadores
	
	public function new_recipe($user, $name, $raw_ingredients, $recipe, $comment, $photo, $data, $edit=null)
	{
		global $config;
		global $nuts;
		$name = substr($name, 0, 50); //tamaño maximo de la celda
		//if (!in_array($difficulty, array('muy facil','facil','normal','dificil','muy dificil')))
		//	$difficulty = 'normal';
	
		$parser = new Nuts_Parser();
	
		$parser->parse($raw_ingredients, 'ingredients')
				->not_empty_html("La receta debe tener al menos un ingrediente")
				;
				
		$parser->parse($recipe, 'recipe')
				->not_empty_html("La receta no puede estar vacía")
				;
				
		$parser->parse(idfy($name), 'namerecipe')
				->minlen(4, "El nombre de la receta debe tener al menos cuatro caracteres")
				;
				
		$parser->parse($name, 'namerecipe')
				->not_empty_html("La receta debe tener un nombre")
				;
				
		//if (!preg_match("/^[\w\ \(\)]+$/u", $name)){dump($name);$GLOBALS['config']['error']['namerecipe'] = "El nombre de la receta no puede contener signos de puntuación";}
		
		if(Parse::is_error()) return; //si hay algun error no continuar
		

		$parser->parse("$user:".idfy($name), 'namerecipe')
				->not_exist('Nuts_MongoRecipe::get_recipe', array($user), "Ya existe una receta con ese nombre");
				
		
		
		if(Parse::is_error()) return; //si hay algun error no continuar
			
		//dump($raw_ingredients);
		//$tag_ingredients = Nuts_Recipe::ingredients2tag($raw_ingredients);
		$tag_ingredients = Nuts_Recipe::tags_in_text($raw_ingredients, $edit);
		$search =$data['tags'] =  array_filter(explode(",", $data['tags']));
		
		$search[] = $user;
		$search[] = @$data['origen'];
		$search[] = @$data['price'];
		$search = array_filter(array_map('idfy', $search));

		$stemm = explode(" ", strip_tags($name));

		$stemm = array_merge($stemm, explode(" ",str_replace("/", " ", implode(" ",$data['tags']))));

		$stemm = array_merge($stemm, explode(" ",implode(" ",$tag_ingredients['mandatory'])));
		$stemm = array_merge($stemm, explode(" ",implode(" ",$tag_ingredients['optional'])));
		$stemm = array_filter(array_map('idfy',$stemm), 'stopword');
		include_once('stemm_es.php');
		$stemm = array_values(array_map('stemm_es::stemm', array_map('trim', $stemm)));
		
		$search = array_values($search);
		//dump($stemm);
		//$search = array();
		//$stemm = array();
		
		$obj = parent::new_recipe($user, $name, $raw_ingredients, $recipe, $comment, $data, $tag_ingredients, $stemm, $search, $edit);

		//si no se intento subir algun archivo no da error
		if ($photo and $photo['error'] != UPLOAD_ERR_NO_FILE) 
		{
			if ($photo['error'] !== UPLOAD_ERR_OK)
					$GLOBALS['nuts']->config['error']['photo'] = "Fallo al cargar la imagen";
				else if ($photo['type'] != "image/jpeg")
					$GLOBALS['nuts']->config['error']['photo'] = "La imagen tiene que estar en formato .jpg";
				else
					if(!move_uploaded_file($photo['tmp_name'], "static/image/recipe/$user.$name.jpg"))
						$GLOBALS['nuts']->config['error']['photo'] = "Fallo al cargar la imagen";
					else
					{ 	
						PRINT "SE DEBERIA CARGAR LA IMAGEN AQUI";
					}
		}
		else //no se cargo una imagen determinada se busca la ultima
		{
			$patrón = '/<img[^>]*src=[\'"]?([^>]*?.jpg)/ui';
			preg_match_all($patrón, $recipe, $coincidencias);
			$filename = end($coincidencias[1]);
			//if($filename[0] == "/")
			//	$filename = "..".$filename;
			if ($filename) //80 * 1,62 = 130 // 100 * 1,62 ~= 160
				{
					$t = $config['noresize'];
					//$config['noresize'] = false;
					include_once('redimensionar.php');
					redimensionar($filename, "static/image/recipe/{$user}.{$obj['id']}.jpg" , 160, 100, false);
					$config['noresize'] = $t;
				}
		}
			
			
		if ($GLOBALS['nuts']->config['error'])
			return; //si hay algun error no continuar

		Nuts_Nuts::successfully($nuts->config['base'] . "user/" . $user . "/recipes/" . urlfy($name));
		return $obj;
	}
	

	public function fav_recipe($recipe, $user)
	{
		if ( parent::is_favorited($recipe, $user) )
			parent::delete_fav_recipe($recipe, $user);
		else 
			parent::add_fav_recipe($recipe, $user);
	}

	public function add_comment($recipe, $user, $comment)
	{
		if (!$comment)
		{
			$GLOBALS['nuts']->config['error']['comment'] = 'El comentario no puden estar vacío';
			return;
		}
		parent::add_comment($recipe, $user, $comment, $info);
		Nuts_Nuts::successfully();
	}
	
	public function add_alternative($user, $recipe, $add, $del, $changes, $comment)
	{
		if (!$changes and !$comment)
		{
			$GLOBALS['config']['error']['comment'] = 'Los campos cambios y comentarios no puden estar vacíos, deberá rellenar al menos uno';
			return;
		}
		parent::add_alternative($user, $recipe, $changes, $comment, $info);
		$idalternative = $info['id'];
		if (!$info['success'])
			return;
		$arr_ingr = Nuts_Recipe::ingredients2array($add);
		
		foreach ($arr_ingr as $ingr)
				parent::add_ingredient_to_alternative($idalternative, $recipe, $ingr[0], $ingr[1]);

		if ($del)
			foreach ($del as $ingredient)
					parent::del_ingredient_to_alternative($idalternative, $recipe, $ingredient);

		Nuts_Nuts::successfully();
	}
	
	//metodo de apoyo
	/**
	* @return array array string con las cantidades y los ingredientes separados
	* @param string $ingredients texto con todos los ingredientes escritos en lineas
	*/
	private function ingredients2array($ingredients)
	{
		//return array();
		$ingrs = explode("\n",$ingredients);
		preg_match_all ( "@<li>(.*?)</li>@u" , $ingredients , $matches);
		$ingrs = $matches[1];
		foreach ( $ingrs as $line)
		{
			$match=array();
			$m=preg_match("/(.*?)\s?\[(.*)\]/u", $line, $match); //200 gr de [mantquilla]; [vino de jerez]
			//$m=preg_match("/(.*?)(?:\sde)?\s?\[(.*)\]/", $line, $match); //200 gr de [mantquilla]; [vino de jerez]
			if(!$m)
				$m=preg_match("/(.*?\sde)\s(.*)/u", $line, $match);
				//$m=preg_match("/(.*?)\sde\s(.*)/", $line, $match);
			if(!$m)
				$m=preg_match("/((?:\d+\/\d+)|\d+|(?:una?\s))?\s?(.*)/u", $line, $match); // (un numero) o ("un, una" seguido de un espacio), y luego opcionalmente un espacio y el ingrediente
			if (trim($match[2]))
				$r[]=array("measure"=>trim($match[1]), "ingredient"=>trim($match[2]));
		}
		return $r;
	}
	
	private function parse_line($line){
		$match=array();
		//dump($line);
		$lines = preg_split("/\bo\b/u", $line); //200 gr de [mantquilla]; [vino de jerez]
		if(count($lines)>1)	return array_map('Nuts_Recipe::parse_line', $lines);
		return $line;
		$m=preg_match("/\s*(.*?)\s*\[(.*)\]\s*/u", $line, $match); //200 gr de [mantquilla]; [vino de jerez]
		//$m=preg_match("/(.*?)(?:\sde)?\s?\[(.*)\]/", $line, $match); //200 gr de [mantquilla]; [vino de jerez]
		if(!$m)
			$m=preg_match("/\s*(.*?\s+de)\s+(.*)\s*/u", $line, $match);
			//$m=preg_match("/(.*?)\sde\s(.*)/", $line, $match);
		if(!$m)
			$m=preg_match("/\s*((?:\d+\/\d+)|\d+|(?:una?\s+))?\s*(.*)\s*/u", $line, $match); // (un numero) o ("un, una" seguido de un espacio), y luego opcionalmente un espacio y el ingrediente
		if (trim($match[2]))
			#$r=array("measure"=>trim($match[1]), "ingredient"=>trim($match[2]));
			$r=trim($match[2]);
		return $r;
	}
	
	function get_all_ingredients($select=array(), $cond=array(), $limit=0){
		$r = &DB::$db->ingredients->find($cond, $select);
		if ($limit) return $r->sort(array('count'=>-1))->limit($limit);
		return $r;
	}
	
	function get_query_ingredients($select=array(), $cond=array(), $limit=0){
		$r = &DB::$db->ingredients->find($cond, $select);
		if ($limit) return $r->sort(array('count'=>-1))->limit($limit);
		return $r;
	}
	
	function found_ingredients($ing){
		$r = &DB::$db->ingredients->update($ing, array('$inc'=>array('count'=>1)));	
	}
	
	private function tags_in_text($ingredients, $edit){
		$all = Nuts_Recipe::get_all_ingredients(array('name'=>1,'s'=>1));
		$ings = Nuts_Recipe::ingredients2tag($ingredients);
		$o = array_filter($ings['optional']);
		$m = $ings['mandatory'];
		$r = array("optional"=>array(),"mandatory"=>array());
		
		foreach($o as $i){
			$find=False;
			$i2 = stemm_text($i);
			foreach ($all as $a)
				if(preg_match("/\b{$a['s']}\b/i", $i2))
				{
					if (!$edit) Nuts_Recipe::found_ingredients($a);
					$r['optional'][] = $a['name'];
					$find=true;
				}
			// if(!$find)dump($i);
		}
		foreach($m as $i){
			$find=False;
			$i2 = stemm_text($i);
			foreach ($all as $a)
				if(preg_match("/\b{$a['s']}\b/i", $i2))
				{
					if (!$edit) Nuts_Recipe::found_ingredients($a);
					$r['mandatory'][] = $a['name'];
					$find=true;
				}
			// if(!$find)dump($i);
		}
		return $r;
	}
	private function ingredients2tag($ingredients)
	{
		$r = array("mandatory"=>array(), "optional"=>array());
		//return array();
		$ingrs = explode("\n",$ingredients);
		preg_match_all ( "@<li>(.*?)</li>@u" , $ingredients , $matches);
		if($matches) $ingrs = $matches[1];
		foreach ($ingrs as $line)
		{	
			//dump(array($line, normalize($line)));
			
			$i = Nuts_Recipe::parse_line(normalize($line));
			if (is_array($i))
				$r['optional'] = array_merge($r['optional'], $i);
			else
				$r['mandatory'][] = $i;
		}
		return $r;
	}
	
	/**
	* Obtener una pagina de las recetas (en la pagina 1 las mas recientes)
	*
	* @return array una pagina de recetas
	* @param int $page numero de la pagina a devolver
	*/
	public function recipes($page=1)
	{
		$init=($page-1)*Nuts_Recipe::$limit;
		return parent::recipes($init, Nuts_Recipe::$limit);
	}
	
	public function page_search($query, $page=1)
	{
		$init=($page-1)*Nuts_Recipe::$limit;
		return parent::search($query, $init, Nuts_Recipe::$limit);
	}
	
	/**
	* Numero de recetas en la base de datos
	*
	* @return int 
	*/
	public function count_recipes()
	{
		return $q = parent::count_recipes();
		return $q['counter'];
	}
	
	
	/**
	* Numero de paginas disponibles
	*
	* @return int 
	*/
	public function number_of_pages()
	{
		return ceil(Nuts_Recipe::count_recipes() / Nuts_Recipe::$limit); //redondeo arriba
	}
	
	/**
	* Obtener las ultimas recetas añadidas
	*
	* @return array informacion de las recetas pedidas
	* @param int $limit numero de recetas pedidas
	*/
	public function last_recipes($limit)
	{
		return parent::recipes(0, $limit);
	}
	
	/**
	* Obtener una receta aleatoria
	*
	* @return array información de la receta
	*/	
	public function random_recipe()
	{
		$recip = mt_rand(1, Nuts_Recipe::count_recipes()-1); //el indice es 1, para que no coincida con la ultima receta y aporte algo nuevo
		return parent::recipes($recip, 1);
	}

	/**
	* Obetner receta mas favorita
	*
	* @return array informacion de la receta
	*/
	public function get_more_favorited()
	{
		return parent::get_more_favorited();
	}
	
	public function public_newRecipe($namerecipe, $ingredients, $recipe, $comment, $original, $preptime, $cooktime, $yield, $draft, $entry,
				$origen, $difficulty, $price, $tags){
				Nuts_Recipe::new_recipe(
					$_SESSION['user']['id'],
					trim($namerecipe), 
					trim($ingredients), 
					trim($recipe),
					trim($comment),
					$_FILES['photo'],
					array(
						"original" => $original,
						"yield" => $yield,
						"draft" => $draft,
						"entry" => $entry,
						"origen" => $origen,
						"difficulty" => $difficulty,
						"price" => $price,
						"tags" => $tags,
						"preptime" => $preptime,
						"cooktime" => $cooktime,
					)
				);	
	}
	
	
	public function public_newRecipe2($namerecipe, $ingredients, $recipe, $comment, $original, $preptime, $cooktime, $yield, $draft, $entry,
				$diettype, $dietrestriction, $origen, $occasion, $kind, $picante, $mode, $type, $difficulty, $price){
				Nuts_Recipe::new_recipe(
					$_SESSION['user']['id'],
					trim($namerecipe), 
					trim($ingredients), 
					trim($recipe), 
					trim($comment),
					$_FILES['photo'],
					array(
						"original" => $original,
						"preptime" => $preptime,
						"cooktime" => $cooktime,
						"yield" => $yield,
						"draft" => $draft,
						"entry" => $entry,
						"diettype" => $diettype,
						"dietrestriction" => $dietrestriction,
						"origen" => $origen,
						"occasion" => $occasion,
						"kind" => $kind,
						"picante" => $picante,
						"mode" => $mode,
						"type" => $type, 
						"difficulty" => $difficulty,
						"price" => $price,
					)
				);	
	}
	public function public_commentRecipe($id, $comment){
		Nuts_Recipe::add_comment($id, $_SESSION['user']['id'], trim($comment));
	}
	public function public_favRecipe($recipe){
		Nuts_Recipe::fav_recipe($recipe, $_SESSION['user']['id']);
	}
	
	public function public_changeImg($recipe, $img){
		$r = explode(':', $recipe);
		if($_SESSION['user']['id'] == $r[0])
			include_once('redimensionar.php');
			redimensionar($img, "static/image/recipe/".(implode('.', $r)).".jpg" , 160, 100, false);
	}

	public function public_editRecipe($namerecipe, $ingredients, $recipe, $comment, $original, $preptime, $cooktime, $yield, $draft, $entry,
				$origen, $difficulty, $price, $tags){
				//dump($_POST, 'edit!');
				
				Nuts_Recipe::new_recipe(
					$_SESSION['user']['id'],
					trim($namerecipe), 
					trim($ingredients), 
					trim($recipe), 
					trim($comment),
					$_FILES['photo'],
					array(
						"original" => $original,
						"preptime" => $preptime,
						"cooktime" => $cooktime,
						"yield" => $yield,
						"draft" => $draft,
						"entry" => $entry,
						"diettype" => $diettype,
						"dietrestriction" => $dietrestriction,
						"origen" => $origen,
						"occasion" => $occasion,
						"kind" => $kind,
						"picante" => $picante,
						"mode" => $mode,
						"type" => $type, 
						"difficulty" => $difficulty,
						"price" => $price,
					),
					'edit'
				);	
	}
	
}

?>